import torch
from diffusers import StableDiffusionPipeline
from GPUtil import showUtilization as gpu_usage
from numba import cuda
import os
import gc
import uuid  # Для генерации уникальных имен файлов


def free_gpu_cache():
    print("Initial GPU Usage")
    gpu_usage()

    torch.cuda.empty_cache()

    cuda.select_device(0)
    cuda.close()
    cuda.select_device(0)

    print("GPU Usage after emptying the cache")
    gpu_usage()


# Создание директории для сохранения изображений
os.makedirs("generated_images", exist_ok=True)

# Загрузка модели
model_id = "runwayml/stable-diffusion-v1-5"
device = "cuda" if torch.cuda.is_available() else "cpu"
if device == 'cuda':
    free_gpu_cache()

pipe = StableDiffusionPipeline.from_pretrained(model_id, torch_dtype=torch.float16)
pipe.to(device)


async def generate_image(text: str) -> str:
    image = pipe(text).images[0]
    unique_filename = f"{uuid.uuid4().hex}.png"  # Генерация уникального имени
    image_path = os.path.join("generated_images", unique_filename)
    image.save(image_path)
    return image_path



from pydantic import BaseModel

from .model import generate_image  # Ensure your model module is correctly imported

from fastapi import FastAPI, Request, Response
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles

from starlette.middleware.cors import CORSMiddleware

app = FastAPI()

# Adding CORS handling
app.add_middleware(
    CORSMiddleware,
    allow_origins=["http://localhost:8000"],  # Specify allowed origins
    allow_credentials=True,
    allow_methods=["*"],  # Allow all methods
    allow_headers=["*"],  # Allow all headers
    expose_headers=["*"],  # Expose all headers
)

# Adding route for serving static files
app.mount("/generated_images", StaticFiles(directory="generated_images"), name="generated_images")


class InputText(BaseModel):
    text: str


@app.get("/", response_class=HTMLResponse)
async def read_root(request: Request):
    return {"request": request}


@app.post("/generate-image/")
async def generate_image_endpoint(input_text: InputText, response: Response):
    image_path = await generate_image(input_text.text)
    response.headers["Access-Control-Allow-Origin"] = "*"
    return {"image_path": image_path}


if __name__ == "__main__":
    import uvicorn

    # Clearly specify that the service will run on localhost:8001
    uvicorn.run(app, host="127.0.0.1", port=80)



import pytest
import os
from fastapi.testclient import TestClient
from ml_service.app.main import app
from ml_service.app.model import generate_image
#
# client = TestClient(app)
#
#
# def test_server_is_up():
#     response = client.get("/")
#     assert response.status_code == 200, f"Expected status code 200, but got {response.status_code}"
    # assert response.json() == {"Hello, World!"}

# def test_generate_image_endpoint():
#     response = client.post("/generate-image/", json={"text": "space pig"})
#     assert response.status_code == 200
#     assert "image_path" in response.json()

# @pytest.mark.asyncio
# async def test_generate_image():
#     text = "space pig"
#     image_path = await generate_image(text)
#     assert os.path.exists(image_path)
# assert image_path == f"generated_images/{text.replace(' ', '_')}.png"



from setuptools import setup, find_packages

setup(
    name="Text-to-Image-Generator",
    version="0.0.1",
    description="A project that combines a FastApi backend with an ML service to generate images from text.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Anna Fomina Melnikova Maria",
    author_email="anna.skachenko@gmail.com",
    url="https://github.com/AnnaFomina1997/Text-to-Image-Generator.git",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "fastapi",
        "jinja2",
        "requests",
        "diffusers",
        "torch",
        "GPUtil",
        "numba",
    ],
    entry_points={
        "console_scripts": [
            "backend = backend.app.main:app",
            "ml_service = ml_service.app.main:app"
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.11',
)
